
//********************************************************************************
//If you should have any comments, suggestions or improvements to these samples,
//we welcome you to contact us at SampleCode@melissadata.com also please visit our
//developers bulletin board at forum.melissadata.com.
//********************************************************************************


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Xml;
using System.Xml.Serialization;
using System.Web;
using System.Net;
using System.IO;

namespace StreetSearchXML
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }


        //*********************************************************************
        // A valid Customer ID is required to access Melissa Data's
        // WebSmart Services. Please call our Sales department
        // at 1-800-635-4772 x3 to request for a valid Customer ID.
        //*********************************************************************

        XmlTextWriter Writer;
        HttpWebRequest request;
        HttpWebResponse response = null;
        Stream responseReader;
        Uri address ;
        Byte[] byteData;
        Stream postStream = null;
        String tServer;
        StreamReader fp;
        String xmlData;
        XmlDocument xmlDoc;
        int currentRecord = 1;
        Boolean isInitializedSS = true;
        DataSet myDataSet;
        DataTable tRes;


        //***************************************************************************
        // Function to process the Street Search request
        //***************************************************************************
        private void btnDoStreetSearch_Click(object sender, EventArgs e)
        {
            //Initialize XML Text Writer
            Writer = new XmlTextWriter("request.xml", null);

            Writer.WriteStartElement("Request");
            Writer.WriteElementString("TransmissionReference", "Testing: DQWS XML Sample Code implementation using single record inputs.");
            Writer.WriteElementString("CustomerID", txtCustomerID.Text);

            if (chkInRangeOnly.Checked)
                Writer.WriteElementString("OptInRangeOnly", "true");
            else
                Writer.WriteElementString("OptInRangeOnly", "false");
       

            Writer.WriteElementString("AddressLine", txtAddress.Text);
            Writer.WriteElementString("City", txtCity.Text);
            Writer.WriteElementString("State", txtState.Text);
            Writer.WriteElementString("Zip", txtZip.Text);
            Writer.WriteElementString("Country", txtCountry.Text);

            Writer.WriteEndElement();
            Writer.Close();

            // Set the Server URL to the form input
        
            tServer = "https://streetsearch.melissadata.net/v2/XML/Service.svc/doStreetSearch";


            address = new Uri(tServer);
        

            // Create the web request   
            request = (HttpWebRequest) WebRequest.Create(address);

            request.Method = "POST";
            request.ContentType = "application/xml; charset=utf-8";

            // Read in XML file and then convert it to string
            fp = File.OpenText("request.xml");
            xmlData = fp.ReadToEnd();
            fp.Close();

            byteData = UTF8Encoding.UTF8.GetBytes(xmlData.ToString());

            // Set the content length in the request headers   
            request.ContentLength = byteData.Length;

            // Write data to a stream
            try
            {
                postStream = request.GetRequestStream();
                postStream.Write(byteData, 0, byteData.Length);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "Error");
                return;
            }
            finally
            {
                if (postStream != null)
                    postStream.Close();
            }

            // Get response   
            response = (HttpWebResponse) request.GetResponse();

            // Get the response stream into a reader   
            responseReader = response.GetResponseStream();

            XmlTextWriter outXML; 
            outXML = new XmlTextWriter("response.xml", null);

            xmlDoc = new XmlDocument();
            xmlDoc.Load(responseReader);
            xmlDoc.Save(outXML);
            Writer.Close();
            outXML.Close();


            // Fill the Individual Fields. Each time verify is hit, must return record counter to 1
            // Must be initialized correctly to continue
            currentRecord = 1;
            isInitializedSS = true;

            PrepareDGView();

            // Clear the Table First 
            dgvResults.Rows.Clear();

            PopulateNodesSS(xmlDoc);

            // Add the Table to the Data Set
            //myDataSet.Tables.Add(tRes);

            //dgvResults.SetDataBinding(myDataSet, "Results");


            webBrowser1.Navigate(System.Environment.CurrentDirectory + "\\request.xml");

            if (isInitializedSS) 
            {
                // View response xml
                webBrowser2.Navigate(System.Environment.CurrentDirectory + "\\response.xml");

                responseReader.Close();
            }
     
        }

        //***************************************************************************
        // Function to initialize the Data Grid Headers
        //***************************************************************************
        public void PrepareDGView()
        {
            if (dgvResults.Columns.Count == 0)
            {
                dgvResults.Columns.Add("Record ID", "Record ID");
                dgvResults.Columns.Add("Company", "Company");
                dgvResults.Columns.Add("Range High", "Range High");
                dgvResults.Columns.Add("Range Low", "Range Low");
                dgvResults.Columns.Add("Range OddEven", "Range OddEven");
                dgvResults.Columns.Add("Street PreDir", "Street PreDir");
                dgvResults.Columns.Add("Street PostDir", "Street PostDir");
                dgvResults.Columns.Add("Street Name", "Street Name");
                dgvResults.Columns.Add("Street Suffix", "Street Suffix");
                dgvResults.Columns.Add("Suite Name", "Suite Name");
                dgvResults.Columns.Add("Suite High", "Suite High");
                dgvResults.Columns.Add("Suite Low", "Suite Low");
                dgvResults.Columns.Add("Suite OddEven", "Suite OddEven");
                dgvResults.Columns.Add("Zip5", "Zip5");
                dgvResults.Columns.Add("Zip+4 High", "Zip+4 High");
                dgvResults.Columns.Add("Zip+4 Low", "Zip+4 Low");
                dgvResults.Columns.Add("Carrier Route", "Carrier Route");
                dgvResults.Columns.Add("Urb Name", "Urb Name");
                dgvResults.Columns.Add("Urb Code", "Urb Code");
                dgvResults.Columns.Add("AddressType", "AddressType");
                dgvResults.Columns.Add("LACS Indicator", "LACS Indicator");
                dgvResults.Columns.Add("BaseAlternateIndicator", "BaseAlternateIndicator");
            }
        }


        //***************************************************************************
        // Functions to Populate The Output Fields
        //***************************************************************************
        // Populate Output Fields that have existing nodes
        public void AddSS(XmlNode theNode)
        {

            if (theNode.ParentNode.Name == "RecordID")
                dgvResults.Rows[currentRecord].Cells["Record ID"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "Company")
                dgvResults.Rows[currentRecord].Cells["Company"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "PreDirection")
                dgvResults.Rows[currentRecord].Cells["Street PreDir"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "PostDirection")
                dgvResults.Rows[currentRecord].Cells["Street PostDir"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "Suffix")
                dgvResults.Rows[currentRecord].Cells["Street Suffix"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "Zip5")
                dgvResults.Rows[currentRecord].Cells["Zip5"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "Plus4Low")
                dgvResults.Rows[currentRecord].Cells["Zip+4 Low"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "Plus4High")
                dgvResults.Rows[currentRecord].Cells["Zip+4 High"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "CarrierRoute")
                dgvResults.Rows[currentRecord].Cells["Carrier Route"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "Code")
                dgvResults.Rows[currentRecord].Cells["Urb Code"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "AddressType")
                dgvResults.Rows[currentRecord].Cells["AddressType"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "LACSIndicator")
                dgvResults.Rows[currentRecord].Cells["LACS Indicator"].Value = theNode.Value;
            else if (theNode.ParentNode.Name == "BaseAlternateIndicator")
                dgvResults.Rows[currentRecord].Cells["BaseAlternateIndicator"].Value = theNode.Value;

            else if (theNode.ParentNode.Name == "Name")
            {
                // Several Nodes use "Name". We must handle and distinguish which name is refering to
                if (theNode.ParentNode.ParentNode.Name == "Street")
                    dgvResults.Rows[currentRecord].Cells["Street Name"].Value = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "Suite")
                    dgvResults.Rows[currentRecord].Cells["Suite Name"].Value = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "Urbanization")
                    dgvResults.Rows[currentRecord].Cells["Urb Name"].Value = theNode.Value;
            }


            else if (theNode.ParentNode.Name == "High")
            {
                // Several Nodes use "Name". We must handle and distinguish which name is refering to
                if (theNode.ParentNode.ParentNode.Name == "PrimaryRange")
                    dgvResults.Rows[currentRecord].Cells["Range High"].Value = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "Suite")
                    dgvResults.Rows[currentRecord].Cells["Suite High"].Value = theNode.Value;
            }


            else if (theNode.ParentNode.Name == "Low")
            {
                // Several Nodes use "Name". We must handle and distinguish which name is refering to
                if (theNode.ParentNode.ParentNode.Name == "PrimaryRange")
                    dgvResults.Rows[currentRecord].Cells["Range Low"].Value = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "Suite")
                    dgvResults.Rows[currentRecord].Cells["Suite Low"].Value = theNode.Value;
            }

            else if (theNode.ParentNode.Name == "OddEven")
            {
                // Several Nodes use "Name". We must handle and distinguish which name is refering to
                if (theNode.ParentNode.ParentNode.Name == "PrimaryRange")
                    dgvResults.Rows[currentRecord].Cells["Range OddEven"].Value = theNode.Value;
                else if (theNode.ParentNode.ParentNode.Name == "Suite")
                    dgvResults.Rows[currentRecord].Cells["Suite OddEven"].Value = theNode.Value;
            }
        }


        //***************************************************************************
        // Populate the Node List through a Recursive Call from the root
        //***************************************************************************
        public void PopulateNodesSS(XmlDocument xml)
        {
            foreach (XmlNode xmlnode in xml)
                SearchRecordSS(xmlnode);

        }

        //***************************************************************************
        // Search for Current Record In the Record List
        //***************************************************************************
        public void SearchRecordSS(XmlNode xml)
        {
            // For Each parentNode As XmlNode In xml.ChildNodes
            foreach (XmlNode xmlNode in xml)
            {
                if ((xmlNode.Name == "Results") && (xmlNode.HasChildNodes))
                {
                    //***********************************************************************
                    // Block to Handle Different Error Codes in Initialization Results
                    String resultCodeString = xmlNode.FirstChild.Value;
                    setResultCodeDefinition(resultCodeString);
                    //***********************************************************************
                }
                else if (xmlNode.Name == "StreetRecord")
                {
                    // Craete new Row
                    dgvResults.Rows.Add();


                    // If The RecordID is equal to the current node being selected, then output all it's sub nodes.
                    if (xmlNode.FirstChild.Name == "RecordID")
                    {
                        currentRecord = Convert.ToInt32(xmlNode.FirstChild.FirstChild.Value) - 1;

                        // Empty Records return 0 recordID. Must be handled
                        if (currentRecord == -1)
                            currentRecord = 0;


                        PopulateSubNodesSS(xmlNode);
                    }

                }
            }
        }


        //***************************************************************************
        // Populate the Node List through a Recursive Call from the root
        //***************************************************************************
        public void PopulateSubNodesSS(XmlNode parentxmlnode)
        {
            foreach (XmlNode node in parentxmlnode)
            {
                AddSS(node);
                PopulateSubNodesSS(node);
            }
        }

        //***************************************************************************
        // Clear input txt fields
        //***************************************************************************
        private void btnClear_Click(object sender, EventArgs e)
        {
            txtAddress.Clear();
            txtCity.Clear();
            txtCountry.Clear();
            txtState.Clear();
            txtZip.Clear();
        }


        //***************************************************************************
        // Define the result codes returned and show it to the results text box.
        //***************************************************************************
        private void setResultCodeDefinition(string resultCodes)
        {

            String[] errorCodeList = null;

            errorCodeList = resultCodes.Split(',');

            foreach (String resultCode in errorCodeList)
            {
                txtSSResults.Text = " ";

                

                if (resultCode == "SE01")
                {
                    txtSSResults.Text += resultCode + ": Web Service Internal Error;  ";
                }
                else if (resultCode == "GE01")
                {
                    txtSSResults.Text += resultCode + ": Empty XML Request Structure;  ";
                }
                else if (resultCode == "GE02")
                {
                    txtSSResults.Text += resultCode + ": Empty XML Request Record Structure;  ";
                }
                else if (resultCode == "GE03")
                {
                    txtSSResults.Text += resultCode + ": Counted records send more than number of records allowed per request;  ";
                }
                else if (resultCode == "GE04")
                {
                    txtSSResults.Text += resultCode + ": CustomerID empty;  ";
                    MessageBox.Show("Please enter your Customer ID", "Invalid Customer ID");
                }
                else if (resultCode == "GE05")
                {
                    txtSSResults.Text += resultCode + ": CustomerID not valid;  ";
                    MessageBox.Show("The Customer ID your entered is invalid. To retrieve a valid " +
                         "Customer ID, please contact a Melissa Data Sales Representative " +
                         "at 800-MELISSA ext. 3 (800-800-6245 ext. 3). ", "Invalid Customer ID");
                }
                else if (resultCode == "GE06")
                {
                    txtSSResults.Text += resultCode + ": CustomerID disabled;  ";
                    MessageBox.Show("The Customer ID your entered has been disabled. " +
                         "Please contact a Melissa Data Sales Representative " +
                         "at 800-MELISSA ext. 3 (800-800-6245 ext. 3). ", "Disabled Customer ID");
                }
                else if (resultCode == "GE07")
                {
                    txtSSResults.Text += resultCode + ": XML Request invalid;  ";
                }

                else if (resultCode == "SS01")
                    txtSSResults.Text += resultCode + ": OptInRangeOnly = True returned 1 or more results;  ";
                else if (resultCode == "SS02")
                    txtSSResults.Text += resultCode + ": OptInRangeOnly = False returned 1 or more results;  ";
                else if (resultCode == "SS03")
                    txtSSResults.Text += resultCode + ": 1000 street records returned but more than 1000 was found. Please narrow your search;  ";
                else if (resultCode == "DE01")
                    txtSSResults.Text += resultCode + ": Required Input Missing;  ";
                else if (resultCode == "DE02")
                    txtSSResults.Text += resultCode + ": No street records found;  ";
            }
        }

    }

   
}